==> org-lsst-ccs-subsystem-utility-main-1.2.5-sources.jar: org/lsst/ccs/subsystem/utility/PduCommonDevice.java <==
package org.lsst.ccs.subsystem.utility;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.drivers.auxelex.PduCommon;
import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.monitor.Device;
import org.lsst.ccs.subsystem.common.ErrorUtils;
import org.lsst.ccs.subsystem.utility.constants.UtilityConfig;

/**
 *  Interface to a SLAC PDU.
 *
 *  This class is designed only to be extended, not instantiated itself
 *
 *  @author Owen Saxton
 */
public class PduCommonDevice extends Device implements SwitchControl {

    /**
     *  Constants.
     */
    protected static final Logger LOG = Logger.getLogger(PduCommonDevice.class.getName());

    protected static final int
        MON_TYPE_VOLTAGE = 0,
        MON_TYPE_CURRENT = 1;

    private static final String
        NODE   = "node";

    private static final Map<String, Integer> mTypeMap = new HashMap<>();
    static {
        mTypeMap.put("VOLTAGE", MON_TYPE_VOLTAGE);
        mTypeMap.put("CURRENT", MON_TYPE_CURRENT);
    }

    /**
     *  Data fields.
     */
    @ConfigurationParameter(name=NODE, category=UtilityConfig.CATEGORY_UTIL, isFinal=true)
    protected volatile Integer node;

    private final PduCommon pdu;
    private final String pduType;
    private final int switchDevice;
    private int validChans;
    private boolean initError = false;


    /**
     *  Constructor.
     *
     *  @param  pdu      The PDU driver object
     *  @param  pduType  The PDU type description
     *  @param  swDevc   The switch device ID
     */
    public PduCommonDevice(PduCommon pdu, String pduType, int swDevc)
    {
        this.pdu = pdu;
        this.pduType = pduType;
        switchDevice = swDevc;
    }


    /**
     *  Performs configuration.
     */
    @Override
    protected void initDevice() {
        if (node == null) {
            ErrorUtils.reportConfigError(LOG, name, NODE, "is missing");
        }
        fullName = pduType + " PDU (" + node + ")";
        validChans = pdu.getValidChannels();
    }


    /**
     *  Performs full initialization.
     */
    @Override
    protected void initialize()
    {
        try {
            pdu.open(node);
            setOnline(true);
            initSensors();
            LOG.log(Level.INFO, "Connected to {0}", fullName);
            initError = false;
        }
        catch (DriverException e) {
            if (!initError) {
                LOG.log(Level.SEVERE, "Error connecting to {0}: {1}", new Object[]{fullName, e});
                initError = true;
            }
        }
    }


    /**
     *  Closes the connection.
     */
    @Override
    protected void close()
    {
        try {
            pdu.close();
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error disconnecting from {0}: {1}", new Object[]{fullName, e});
        }
    }


    /**
     *  Checks a monitoring channel's parameters for validity.
     *
     *  @param  name     The channel name
     *  @param  hwChan   The hardware channel
     *  @param  type     The channel type string
     *  @param  subtype  The channel subtype string
     *  @return  Two-element array containing the encoded type [0] and subtype [1] 
     *  @throws  Exception  If parameters are invalid
     */
    @Override
    protected int[] checkChannel(String name, int hwChan, String type, String subtype) throws Exception
    {
        Integer mType = mTypeMap.get(type.toUpperCase());
        if (mType == null) {
            ErrorUtils.reportChannelError(LOG, name, "type", type);
        }
        if ((mType == MON_TYPE_VOLTAGE || mType == MON_TYPE_CURRENT)
              && (hwChan < 0 || ((1 << hwChan) & validChans) == 0)) {
            ErrorUtils.reportChannelError(LOG, name, "hwChan", hwChan);
        }
        return new int[]{mType, 0};
    }


    /**
     *  Reads a monitoring channel.
     *
     *  @param  hwChan  The hardware channel number
     *  @param  type    The encoded channel type
     *  @return  The read value
     */
    @Override
    protected double readChannel(int hwChan, int type)
    {
        double value = Double.NaN;
        String item = null;
        if (online) {
            try {
                switch (type) {
                case MON_TYPE_VOLTAGE:
                    item = "voltage";
                    value = pdu.readVoltage(hwChan);
                    break;

                case MON_TYPE_CURRENT:
                    item = "current";
                    value = pdu.readCurrent(hwChan);
                    break;
                }
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error reading {0} {1}: {2}", new Object[]{fullName, item, e});
                setOnline(false);
            }
        }
        return value;
    }


    /**
     *  Gets the switch device type
     * 
     *  @return  The switch device type
     */
    @Override
    public int getSwitchDevice()
    {
        return switchDevice;
    }


    /**
     *  Turns a PDU channel on.
     *
     *  @param  chan  The channel number
     */
    @Override
    public void switchOn(int chan)
    {
        if (chan < 0) {
            try {
                pdu.setMainOn();
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting main power on {0}: {1}", new Object[]{fullName, e});
            }
        }
        else {
            try {
                pdu.powerOn(chan);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting channel {0} on {1}: {2}", new Object[]{chan, fullName, e});
            }
        }
    }


    /**
     *  Turns a PDU channel off.
     *
     *  @param  chan  The channel number
     */
    @Override
    public void switchOff(int chan)
    {
        if (chan < 0) {
            try {
                pdu.setMainOff();
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting main power on {0}: {1}", new Object[]{fullName, e});
            }
        }
        else {
            try {
                pdu.powerOff(chan);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting channel {0} on {1}: {2}", new Object[]{chan, fullName, e});
            }
        }
    }


    /**
     *  Gets the state of a PDU channel.
     *
     *  @param  chan  The channel number
     *  @return  Whether the channel is on or off
     */
    @Override
    public Boolean isSwitchOn(int chan)
    {
        if (!online) return null;
        if (chan < 0) {
            try {
                return pdu.isMainOn();
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error getting state for main power on {0}: {1}", new Object[]{fullName, e});
                return null;
            }
        }
        else {
            if ((validChans & (1 << chan)) == 0) return null;
            try {
                return pdu.isPowered(chan);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error getting state for channel {0} on {1}: {2}", new Object[]{chan, fullName, e});
                return null;
            }
        }
    }

}

==> org-lsst-ccs-subsystem-utility-main-1.3.0-sources.jar: org/lsst/ccs/subsystem/utility/PduCommonDevice.java <==
package org.lsst.ccs.subsystem.utility;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.drivers.auxelex.PduCommon;
import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.monitor.Device;
import org.lsst.ccs.subsystem.common.ErrorUtils;
import org.lsst.ccs.subsystem.utility.constants.UtilityConfig;

/**
 *  Interface to a SLAC PDU.
 *
 *  This class is designed only to be extended, not instantiated itself
 *
 *  @author Owen Saxton
 */
public class PduCommonDevice extends Device implements SwitchControl {

    /**
     *  Constants.
     */
    protected static final Logger LOG = Logger.getLogger(PduCommonDevice.class.getName());

    protected static final int
        MON_TYPE_VOLTAGE = 0,
        MON_TYPE_CURRENT = 1;

    private static final String
        NODE   = "node";

    private static final Map<String, Integer> mTypeMap = new HashMap<>();
    static {
        mTypeMap.put("VOLTAGE", MON_TYPE_VOLTAGE);
        mTypeMap.put("CURRENT", MON_TYPE_CURRENT);
    }

    /**
     *  Data fields.
     */
    @ConfigurationParameter(name=NODE, category=UtilityConfig.CATEGORY_UTIL, isFinal=true)
    protected volatile Integer node;

    private final PduCommon pdu;
    private final String pduType;
    private final int switchDevice;
    private int validChans;
    private boolean initError = false;


    /**
     *  Constructor.
     *
     *  @param  pdu      The PDU driver object
     *  @param  pduType  The PDU type description
     *  @param  swDevc   The switch device ID
     */
    public PduCommonDevice(PduCommon pdu, String pduType, int swDevc)
    {
        this.pdu = pdu;
        this.pduType = pduType;
        switchDevice = swDevc;
    }


    /**
     *  Performs configuration.
     */
    @Override
    protected void initDevice() {
        if (node == null) {
            ErrorUtils.reportConfigError(LOG, name, NODE, "is missing");
        }
        fullName = path + " (" + pduType + " PDU at node " + node + ")";
        validChans = pdu.getValidChannels();
    }


    /**
     *  Performs full initialization.
     */
    @Override
    protected void initialize()
    {
        try {
            pdu.open(node);
            setOnline(true);
            initSensors();
            LOG.log(Level.INFO, "Connected to {0}", fullName);
            initError = false;
        }
        catch (DriverException e) {
            if (!initError) {
                LOG.log(Level.SEVERE, "Error connecting to {0}: {1}", new Object[]{fullName, e});
                initError = true;
            }
        }
    }


    /**
     *  Closes the connection.
     */
    @Override
    protected void close()
    {
        try {
            pdu.close();
        }
        catch (DriverException e) {
            LOG.log(Level.SEVERE, "Error disconnecting from {0}: {1}", new Object[]{fullName, e});
        }
    }


    /**
     *  Checks a monitoring channel's parameters for validity.
     *
     *  @param  name     The channel name
     *  @param  hwChan   The hardware channel
     *  @param  type     The channel type string
     *  @param  subtype  The channel subtype string
     *  @return  Two-element array containing the encoded type [0] and subtype [1] 
     *  @throws  Exception  If parameters are invalid
     */
    @Override
    protected int[] checkChannel(String name, int hwChan, String type, String subtype) throws Exception
    {
        Integer mType = mTypeMap.get(type.toUpperCase());
        if (mType == null) {
            ErrorUtils.reportChannelError(LOG, name, "type", type);
        }
        if ((mType == MON_TYPE_VOLTAGE || mType == MON_TYPE_CURRENT)
              && (hwChan < 0 || ((1 << hwChan) & validChans) == 0)) {
            ErrorUtils.reportChannelError(LOG, name, "hwChan", hwChan);
        }
        return new int[]{mType, 0};
    }


    /**
     *  Reads a monitoring channel.
     *
     *  @param  hwChan  The hardware channel number
     *  @param  type    The encoded channel type
     *  @return  The read value
     */
    @Override
    protected double readChannel(int hwChan, int type)
    {
        double value = Double.NaN;
        String item = null;
        if (isOnline()) {
            try {
                switch (type) {
                case MON_TYPE_VOLTAGE:
                    item = "voltage";
                    value = pdu.readVoltage(hwChan);
                    break;

                case MON_TYPE_CURRENT:
                    item = "current";
                    value = pdu.readCurrent(hwChan);
                    break;
                }
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error reading {0} {1}: {2}", new Object[]{fullName, item, e});
                setOnline(false);
            }
        }
        return value;
    }


    /**
     *  Gets the switch device type
     * 
     *  @return  The switch device type
     */
    @Override
    public int getSwitchDevice()
    {
        return switchDevice;
    }


    /**
     *  Turns a PDU channel on.
     *
     *  @param  chan  The channel number
     */
    @Override
    public void switchOn(int chan)
    {
        if (chan < 0) {
            try {
                pdu.setMainOn();
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting main power on {0}: {1}", new Object[]{fullName, e});
            }
        }
        else {
            try {
                pdu.powerOn(chan);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting channel {0} on {1}: {2}", new Object[]{chan, fullName, e});
            }
        }
    }


    /**
     *  Turns a PDU channel off.
     *
     *  @param  chan  The channel number
     */
    @Override
    public void switchOff(int chan)
    {
        if (chan < 0) {
            try {
                pdu.setMainOff();
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting main power on {0}: {1}", new Object[]{fullName, e});
            }
        }
        else {
            try {
                pdu.powerOff(chan);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error setting channel {0} on {1}: {2}", new Object[]{chan, fullName, e});
            }
        }
    }


    /**
     *  Gets the state of a PDU channel.
     *
     *  @param  chan  The channel number
     *  @return  Whether the channel is on or off
     */
    @Override
    public Boolean isSwitchOn(int chan)
    {
        if (!isOnline()) return null;
        if (chan < 0) {
            try {
                return pdu.isMainOn();
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error getting state for main power on {0}: {1}", new Object[]{fullName, e});
                return null;
            }
        }
        else {
            if ((validChans & (1 << chan)) == 0) return null;
            try {
                return pdu.isPowered(chan);
            }
            catch (DriverException e) {
                LOG.log(Level.SEVERE, "Error getting state for channel {0} on {1}: {2}", new Object[]{chan, fullName, e});
                return null;
            }
        }
    }

}
